;=============================================================================
; FILE      = spl78k0_kx2.asm
; PROJECT   = selfprogramming application note, MF2 single voltage flash
; DEVICE    = 78k0/Kx2
; VERSION   = 1v06
; DATE      = 12.09.2006 09:29   (c)2004-2006 by NEC Electronics (Europe) GmbH
;=============================================================================
; Purpose:      implementation of the low-level access flash memory access
;=============================================================================
; Enviroment:   IAR for 78K0 and 78K0S (version V3.xx and V4.xx)
;=============================================================================

 NAME           spl78k0_kx2             ; module name

                                        ;======================================
                                        ; INCLUDE LIST
                                        ;======================================
#include  "spl78k0_kx2_config.h"        ; NEC-EE
#include  "spl78k0_kx2_const.h"         ; NEC-EE

                                        ;======================================
                                        ; EXPORT LIST
                                        ;======================================
                                        ; Constants:
 PUBLIC   SPL_INFO                      ;  PUBLIC for device and version control
                                        ; Variables:
 PUBLIC   entry_ram                     ;  PUBLIC for EEPROM emulation purpose only
                                        ; Flags:
                                        ; Labels:
 PUBLIC   SelfLib_Init                  ; u08 SelfLib_Init(u08* data_buffer_pu08);
 PUBLIC   SelfLib_ModeCheck             ; u08 SelfLib_ModeCheck(void);
 PUBLIC   SelfLib_BlankCheck            ; u08 SelfLib_BlankCheck(u08 block_u08);
 PUBLIC   SelfLib_Erase                 ; u08 SelfLib_Erase(u08 block_u08);
 PUBLIC   SelfLib_Verify                ; u08 SelfLib_Verify(u08 block_u08);
 PUBLIC   SelfLib_Write                 ; u08 SelfLib_Write(u32 s_address_u32, u08 wordcount_u08);
 PUBLIC   SelfLib_EepWrite              ; u08 SelfLib_EepWrite(u32 s_address_u32, u08 wordcount_u08);
 PUBLIC   ASM_execute                   ; PUBLIC for EEPROM emulation purpose only
                                        ;======================================


                                        ;======================================
                                        ; IMPORT LIST
                                        ;======================================
                                        ; Constants:
                                        ; Variables:
                                        ; Flags:
                                        ; Labels:


; temporary abbreviation for conditional assembly
#define   GI_S    (GetInfo_Security==spl78k0_function_included)
#define   GI_BC   (GetInfo_BootCluster==spl78k0_function_included)
#define   GI_BEA  (GetInfo_BlockEndAddr==spl78k0_function_included)
#define   SI_SBC  (SetInfo_SwapBootCluster==spl78k0_function_included)
#define   SI_CEP  (SetInfo_ChipEraseProtection==spl78k0_function_included)
#define   SI_BEP  (SetInfo_BlockEraseProtection==spl78k0_function_included)
#define   SI_WP   (SetInfo_WriteProtection==spl78k0_function_included)
#define   SI_BCP  (SetInfo_BootClusterProtection==spl78k0_function_included)



;=============================================================================
; Segment name:  DS_ERAM
; Segment type:  self-lib data-segment
; Address area:  location corresponds with XCL-File
;=============================================================================
; The Enrty-RAM can be located in high-speed RAM as well as in short-address RAM.
; When located in saddr area the real-time behavior of selfprogramming improves.
;=============================================================================
 RSEG           DS_ERAM (1)
;=============================================================================
entry_ram      DS       ENTRY_RAM_SIZE



;=============================================================================
; Segment name:  BCLUST0
; Segment type:  self-lib code segment inside the boot-claster 0 (block 0..3)
; Address area:  internal ROM area 0082H-0FFFH
;=============================================================================
; The code of the selfprogramming library have to be located in the active
; boot-cluster area 0x0000...0x0FFF. Before swapping the boot-cluster the
; application should take care for valid boot-loader inside the activated .
;=============================================================================
 RSEG          BCLUST0 (1)
;=============================================================================


;-----------------------------------------------------------------------------
; Block type:   selflib command function
; C prototype:  void SelfLib_Init(u08* data_buffer_pu08);
; C example:    SelfLib_Init((u08*)&my_data_buffer[0]);
;-----------------------------------------------------------------------------
; Purpose:      initialization of internal selfprogramming environment.
;               After initialization the pointer to the data-buffer is stored
;               in the EntryRAM and the block-erase retry-counter is downsized
;               from 255 (firmware default value) to ERASE_RETRY_COUNTER
;               defined in spl78k0_kx2_const.h. If other value required, the
;               user can reconfigure this constant.
; Input:        AX, data_buffer_pu08 - pointer to a data buffer N...256 bytes
;               (used for data exchange between firmware and application)
; Output:       -
; Return:       -
;-----------------------------------------------------------------------------
SelfLib_Init:
    PUSH    HL                                      ; save used register
    PUSH    AX                                      ; keep data_buffer_pu08 "in mind"
    MOV     A,#CMD_INIT                             ; select the command
    CALL    ASM_execute                             ; execute the command
    MOVW    HL,AX                                   ; keep FW status "in mind"
    POP     AX                                      ; get data_buffer_pu08
    MOVW    (entry_ram+IDX_DATA_BUF_H),AX           ; set address of data_buffer
    MOV     A,#ERASE_RETRY_COUNTER                  ;
    MOV     (entry_ram+IDX_ERASE_RETRY_COUNTER),A   ; reduce the erase retry counter 255->N
    MOVW    AX,HL                                   ; restore FW status
    POP     HL                                      ; restore used register
    RET                                             ;



;-----------------------------------------------------------------------------
; Block type:   selflib command function
; C prototype:  u08 SelfLib_ModeCheck(void);
; C example:    my_status_u08 = SelfLib_ModeCheck();
;-----------------------------------------------------------------------------
; Purpose:      checks the voltage level at FLMD0 pin.
; Input:        -
; Output:       -
; Returned:     A, status code
;                  = 0x00(STS_NO_ERROR), normal and means FLMD0=HIGH
;                  = 0x01(STS_ERROR), error, FLMD0=LOW
;-----------------------------------------------------------------------------
SelfLib_ModeCheck:
    MOV     A,#CMD_CHECK_FLMD                 ; select the command
    CALL    ASM_execute                       ; execute the command
    RET                                       ; and return the status to the caller



;-----------------------------------------------------------------------------
; Block type:   selflib command function
; C prototype:  u08 SelfLib_BlankCheck(u08 block_u08);
; C example:    my_status_u08 = SelfLib_BlankCheck(my_block_u08);
;-----------------------------------------------------------------------------
; Purpose:      checks if specified block is blank
; Input:        A, block_u08 - block number has to be checked
; Output:       -
; Returned:     A, status code
;                  = 0x00(STS_NO_ERROR), normal and means "block is blank"
;                  = 0x05(STS_PARAMETER_ERROR), parameter error (wrong block nr)
;                  = 0x1B(STS_MRG11_ERROR), blank-check error means "block not blank"
;                  = 0x1F(STS_INTERRUPTED), blank-check interrupted by user interrupt
;-----------------------------------------------------------------------------
SelfLib_BlankCheck:
    MOV     (entry_ram+IDX_BLOCK_NO),A        ; entry_ram[IDX_BLOCK_NO] <- my_block_u08
    MOV     A,#CMD_BLANKCHECK_BLOCK           ; RB(user).A <- command code
    CALL    ASM_execute                       ; execute the command
    RET                                       ;



;-----------------------------------------------------------------------------
; Block type:   selflib command function
; C prototype:  u08 SelfLib_Erase(u08 block_u08);
; C example:    my_status_u08 = SelfLib_Erase(my_block_u08);
;-----------------------------------------------------------------------------
; Purpose:      erase specified block
; Input:        A, block_u08 - block number has to be erase
; Output:       -
; Returned:     A, status_code
;                  = 0x00(STS_NO_ERROR), normal and means "block erased successfully"
;                  = 0x05(STS_PARAMETER_ERROR), parameter error
;                  = 0x10(STS_PROTECTION_ERROR), tried to erase protected boot-cluster
;                  = 0x1A(STS_MRG10_ERROR), erase error, retry up to max. 255 times
;                  = 0x1F(STS_INTERRUPTED), erasing interrupted by user interrupt
;-----------------------------------------------------------------------------
SelfLib_Erase:
    MOV     (entry_ram+IDX_BLOCK_NO),A        ; entry_ram[IDX_BLOCK_NO] <- my_block_u08
    MOV     A,#CMD_ERASE_BLOCK                ; RB(user).A <- command code
    CALL    ASM_execute                       ; execute the command
    RET                                       ;



;-----------------------------------------------------------------------------
; Block type:   selflib command function
; C prototype:  u08 SelfLib_Verify(u08 block_u08);
; C example:    my_status_u08 = SelfLib_Verify(my_block_u08);
;-----------------------------------------------------------------------------
; Purpose:      performs internal verify of specified block
; Input:        A, block_u08 - block number has to be verified
; Output:       -
; Returned:     A, status_code
;                  = 0x00(STS_NO_ERROR), normal and means "block is verified"
;                  = 0x05(STS_PARAMETER_ERROR), parameter error
;                  = 0x1B(STS_MRG11_ERROR), internal verify error
;                  = 0x1F(STS_INTERRUPTED), verify interrupted by user interrupt
;-----------------------------------------------------------------------------
SelfLib_Verify:
    MOV     (entry_ram+IDX_BLOCK_NO),A        ; entry_ram[IDX_BLOCK_NO] <- my_block_u08
    MOV     A,#CMD_VERIFY_BLOCK               ; RB(user).A <- command code
    CALL    ASM_execute                       ; execute the command
    RET                                       ;



;-----------------------------------------------------------------------------
; Block type:   selflib command function
; C prototype:  u08  SelfLib_Write   (u32 s_address_u32, u08 word_count_u08)
;               u08  SelfLib_EepWrite(u32 s_address_u32, u08 word_count_u08)
;-----------------------------------------------------------------------------
; Purpose:      writes N words to a specified flash address (multiple of 4)
; Parameter:    see function prototype description in spl78k0_kx2.h
;-----------------------------------------------------------------------------
; Parameter passing:
; IAR 3.xx:     s_address_u32 in AX/BC, word_count_u08 in Stack[SP+4]
; IAR 4.xx:     s_address_u32 in AX/BC, word_count_u08 in D
;-----------------------------------------------------------------------------
SelfLib_Write:                                ; CAUTION !!!!
    MOV     B,#CMD_WRITE_WORD                 ; register B not PUSH'ed here
    BR      SelfLib_Wr_Common                 ; because real flash addr 24 bit.
                                              ; Consequently B=MSB(addr) remains unused

SelfLib_EepWrite:
    MOV     B,#CMD_EEP_WRITE_WORD             ;

SelfLib_Wr_Common:
    MOV     (entry_ram+IDX_START_ADDR_M),A    ; entry_ram[IDX_START_ADDR_M] <- middle byte
    XCH     A,X                               ;
    MOV     (entry_ram+IDX_START_ADDR_L),A    ; entry_ram[IDX_START_ADDR_L] <- low byte
    XCH     A,C                               ;
    MOV     (entry_ram+IDX_START_ADDR_H),A    ; entry_ram[IDX_START_ADDR_H] <- high byte

#if __VER__ >= 400
    ; IAR 4.xx: fetch "word_count_u08"
    ; from register D
    MOV     A,D                               ;
    MOV     (entry_ram+IDX_WORD_COUNT),A      ; entry_ram[IDX_WORD_COUNT] <- word count
#else
    ; IAR 3.xx: fetch "word_count_u08"
    ;from the stack
    PUSH    HL                                ; save used register
    MOVW    AX,SP                             ;
    XCHW    AX,HL                             ;
    MOV     A,[HL+4]                          ; A <- word_count_u08
    MOV     (entry_ram+IDX_WORD_COUNT),A      ; entry_ram[IDX_WORD_COUNT] <- word count
#endif

    ; entry-ram prepared ->> execute the command
    MOV     A,B                               ; RB(user).A <- command code
    CALL    ASM_execute                       ; execute the command

#if __VER__ >= 400
    ; IAR 4.xx: firmware status already in register A
    RET                                       ;
#else
    ; IAR 3.xx: reconstruct the stack and SP before RET

    ; shift the write RET-addr. iside the stack
    ; AX overwritten here, but it will be recovered directly from RB3 later
    MOV     A,[HL+2]                          ; 1v04, A <- PC(7...0)
    MOV     [HL+4],A                          ; 1v04, SP[6] <- SP[4]
    MOV     A,[HL+3]                          ; 1v04, A <- PC(15...8)
    MOV     [HL+5],A                          ; 1v04, SP[7] <- SP[5]
    POP     HL                                ; 1v04, restore used register
    POP     AX                                ; 1v04, stack-pointer correction (-2) (caused at caller side)
    MOV     A,0FEE3H                          ; 1v04, recover the status, RB(user).A <- status code (RB3.B)
    RET                                       ;
#endif


#if (GI_S)
  PUBLIC   SelfLib_GetInfo_Security
  ;-----------------------------------------------------------------------------
  ; Block type:   selflib command function
  ; C prototype:  u08  SelfLib_GetInfo_Security(u08 *destination_pu08)
  ; C example:    my_status_u08 =  SelfLib_GetInfo_Security(my_destination_pu08);
  ;-----------------------------------------------------------------------------
  ; Purpose:      reads the security information (write-/erase-protection)
  ; Input:        AX, destination_pu08 - destination address of the security info
  ;               The format of the security info is: "unsigned char"
  ; Output:       -
  ; Changed:      X, content of the data_buffer
  ; Returned:     A, status code
  ;                  = 0x00(STS_NO_ERROR), normal
  ;                  = 0x05(STS_PARAMETER_ERROR), parameter error
  ;-----------------------------------------------------------------------------
  SelfLib_GetInfo_Security:
      PUSH    BC                                ; save used register
      PUSH    AX                                ; stack <- destination address
      MOV     A,#OPT_SECURITY_INFO              ; A <- option byte value
      MOV     C,#1                              ; C <- length of the info
      BR      SelfLib_GI_Execute
#endif


#if (GI_BC)
  PUBLIC   SelfLib_GetInfo_BootCluster
  ;-----------------------------------------------------------------------------
  ; Block type:   selflib command function
  ; C prototype:  u08  SelfLib_GetInfo_BootCluster(u08 *destination_pu08)
  ; C example:    my_status_u08 =  SelfLib_GetInfo_BootCluster((u08*)&my_destination_u08);
  ;-----------------------------------------------------------------------------
  ; Purpose:      read the current value of the boot flag in extra area
  ; Input:        AX, destination_pu08 - destination address of the bootflag info
  ;               The format of the boot-flag info is: "unsigned char"
  ;               The value of the boot info is 0x00 for cluster 0 and 0x01 for cluster 1.
  ; Output:       -
  ; Changed:      X, content of the data_buffer
  ; Returned:     A, status code
  ;                  = 0x00(STS_NO_ERROR), normal
  ;                  = 0x05(STS_PARAMETER_ERROR), parameter error
  ;-----------------------------------------------------------------------------
  SelfLib_GetInfo_BootCluster:
      PUSH    BC                                ; save used register
      PUSH    AX                                ; stack <- destination address
      MOV     A,#OPT_BOOTCLUSTER_INFO           ; A <- option byte value
      MOV     C,#1                              ; C <- length of the info
      BR      SelfLib_GI_Execute
#endif


#if (GI_BEA)
  PUBLIC   SelfLib_GetInfo_BlockEndAddr
  ;-----------------------------------------------------------------------------
  ; Block type:   selflib command function
  ; C prototype:  u08  SelfLib_GetInfo_BlockEndAddr((u32*) destination_pu32, u08 block_u08)
  ; C example:    my_status_u08 =  SelfLib_GetInfo_BlockEndAddr((u32*)&my_destination_u32, my_block_u08);
  ;-----------------------------------------------------------------------------
  ; Purpose:      puts the last address of the specified block into *destination_pu32
  ; Input:        AX - addr where the block end-address should be stored
  ;               The type is u32 in spite of the fact, that the real flash address is 24 bit.
  ;               B - block number the end-address is asked for.
  ; Output:       -
  ; Changed:      -
  ; Returned:     A, status code
  ;                  = 0x00(STS_NO_ERROR), normal
  ;                  = 0x05(STS_PARAMETER_ERROR), parameter error
  ;-----------------------------------------------------------------------------
  SelfLib_GetInfo_BlockEndAddr:
      PUSH    BC                                ; save used register
      PUSH    AX                                ; stack <- destination address
      PUSH    HL                                ; save used register
      MOV     A,B                               ;
      MOV     (entry_ram+IDX_BLOCK_NO_INFO),A   ; entry_ram[IDX_BLOCK_NO_INFO] <- my_block_u08
      MOVW    AX,(entry_ram+IDX_DATA_BUF_H)     ; AX <- address of data_buffer
      MOVW    HL,AX                             ;
      MOV     A,#00H                            ;
      MOV     [HL+3],A                          ; clear address bits 31...24 in advance
      POP     HL                                ; restore used register
      MOV     A,#OPT_LASTADDRESS_INFO           ; A <- option byte value
      MOV     C,#4                              ; C <- length of the info
#endif


#if (GI_S OR GI_BC OR GI_BEA)
  ;============================================================================
  ; Common part of all GetInfo functions
  ;============================================================================
  SelfLib_GI_Execute:
      MOV     (entry_ram+IDX_OPTION_BYTE),A     ; entry_ram[IDX_OPTION_BYTE] <- act. option byte
      MOV     A,#CMD_GET_INFO                   ;
      CALL    ASM_execute                       ; execute the "get info" command
      POP     HL                                ; HL <- destination address
      CMP     A,#STS_NO_ERROR                   ;
      BNZ     SelfLib_GI_Neg_Exit               ; if(error) goto error_exit;
                                                ;
  SelfLib_GI_Pos_Exit:                          ; else   {
                                                ;   --------------------------------------------
                                                ;   copy information data_buffer[]->&destination
                                                ;   --------------------------------------------
                                                ;   HL = destination
                                                ;   C = number of bytes
                                                ;   --------------------------------------------
      PUSH    AX                                ;   save the status
      PUSH    DE                                ;   save used registers
      MOVW    AX,(entry_ram+IDX_DATA_BUF_H)     ;
      MOVW    DE,AX                             ;   DE <- &(data buffer)

  SelfLib_GI_copy:                              ;
      MOV     A,[DE]                            ;   A <- info byte
      MOV     [HL],A                            ;   destination <- info byte
      INCW    HL                                ;   HL++
      INCW    DE                                ;   DE++
      DBNZ    C,SelfLib_GI_copy                 ;
      POP    DE                                 ;   restore used register
      POP    AX                                 ;   restore used register
                                                ;   --------------------------------------------
  SelfLib_GI_Neg_Exit:                          ; }
      POP    BC                                 ; restore used register
      RET                                       ;
#endif


#if (SI_SBC)
  PUBLIC   SelfLib_SetInfo_SwapBootCluster
  ;-----------------------------------------------------------------------------
  ; Block type:   selflib command function
  ; C prototype:  u08  SelfLib_SetInfo_SwapBootCluster(void);
  ; C example:    my_status_u08 =  SelfLib_SetInfo_SwapBootCluster();
  ;-----------------------------------------------------------------------------
  ; Purpose:      inverts the current value of the boot flag in the extra area
  ; Input:        -
  ; Output:       -
  ; Returned:     u08 - status code
  ;                   = 0x00(STS_NO_ERROR), normal
  ;                   = 0x05(STS_PARAMETER_ERROR), parameter error
  ;                   = 0x10(STS_PROTECTION_ERROR), protection error
  ;                   = 0x1A(STS_MRG10_ERROR), erase error
  ;                   = 0x1B(STS_MRG11_ERROR), internal verify error
  ;                   = 0x1F(STS_INTERRUPTED), write interrupted by user interrupt
  ;-----------------------------------------------------------------------------
  SelfLib_SetInfo_SwapBootCluster:              ;
      PUSH    BC                                ; push used register
      MOV     B,#MASK_SWAP_BOOTCLUSTER          ; B <- security information
      BR      SelfLib_SetInfo_common            ; goto common part of SetInfo
                                                ; -----------------------------------
#endif



#if (SI_CEP)
  PUBLIC   SelfLib_SetInfo_ChipEraseProtection
  ;-----------------------------------------------------------------------------
  ; Block type:   selflib command function
  ; C prototype:  u08  SelfLib_SetInfo_ChipEraseProtection(void);
  ; C example:    my_status_u08 =  SelfLib_SetInfo_ChipEraseProtection();
  ;-----------------------------------------------------------------------------
  ; Purpose:      sets the chip-erase-protection flag in the extra area
  ; Input:        -
  ; Output:       -
  ; Returned:     u08 - status code
  ;                   = 0x00(STS_NO_ERROR), normal
  ;                   = 0x05(STS_PARAMETER_ERROR), parameter error
  ;                   = 0x10(STS_PROTECTION_ERROR), protection error
  ;                   = 0x1A(STS_MRG10_ERROR), erase error
  ;                   = 0x1B(STS_MRG11_ERROR), internal verify error
  ;                   = 0x1F(STS_INTERRUPTED), write interrupted by user interrupt
  ;-----------------------------------------------------------------------------
  SelfLib_SetInfo_ChipEraseProtection:          ;
      PUSH    BC                                ; push used register
      MOV     B,#MASK_CHIP_ERASE_PROTECTION     ; B <- security information
      BR      SelfLib_SetInfo_common            ; goto common part of SetInfo
                                                ; -----------------------------------
#endif



#if (SI_BEP)
  PUBLIC   SelfLib_SetInfo_BlockEraseProtection
  ;-----------------------------------------------------------------------------
  ; Block type:   selflib command function
  ; C prototype:  u08  SelfLib_SetInfo_BlockEraseProtection(void);
  ; C example:    my_status_u08 =  SelfLib_SetInfo_BlockEraseProtection();
  ;-----------------------------------------------------------------------------
  ; Purpose:      sets the block-erase-protection flag in the extra area
  ; Input:        -
  ; Output:       -
  ; Returned:     u08 - status code
  ;                   = 0x00(STS_NO_ERROR), normal
  ;                   = 0x05(STS_PARAMETER_ERROR), parameter error
  ;                   = 0x10(STS_PROTECTION_ERROR), protection error
  ;                   = 0x1A(STS_MRG10_ERROR), erase error
  ;                   = 0x1B(STS_MRG11_ERROR), internal verify error
  ;                   = 0x1F(STS_INTERRUPTED), write interrupted by user interrupt
  ;-----------------------------------------------------------------------------
  SelfLib_SetInfo_BlockEraseProtection:         ;
      PUSH    BC                                ; push used register
      MOV     B,#MASK_BLOCK_ERASE_PROTECTION    ; B <- security information
      BR      SelfLib_SetInfo_common            ; goto common part of SetInfo
                                                ; -----------------------------------
#endif



#if (SI_WP)
  PUBLIC   SelfLib_SetInfo_WriteProtection
  ;-----------------------------------------------------------------------------
  ; Block type:   selflib command function
  ; C prototype:  u08  SelfLib_SetInfo_WriteProtection(void);
  ; C example:    my_status_u08 =  SelfLib_SetInfo_WriteProtection();
  ;-----------------------------------------------------------------------------
  ; Purpose:      sets the write-protection flag in the extra area
  ; Input:        -
  ; Output:       -
  ; Returned:     u08 - status code
  ;                   = 0x00(STS_NO_ERROR), normal
  ;                   = 0x05(STS_PARAMETER_ERROR), parameter error
  ;                   = 0x10(STS_PROTECTION_ERROR), protection error
  ;                   = 0x1A(STS_MRG10_ERROR), erase error
  ;                   = 0x1B(STS_MRG11_ERROR), internal verify error
  ;                   = 0x1F(STS_INTERRUPTED), write interrupted by user interrupt
  ;-----------------------------------------------------------------------------
  SelfLib_SetInfo_WriteProtection:              ;
      PUSH    BC                                ; push used register
      MOV     B,#MASK_WRITE_PROTECTION          ; B <- security information
      BR      SelfLib_SetInfo_common            ; goto common part of SetInfo
                                                ; -----------------------------------
#endif



#if (SI_BCP)
  PUBLIC   SelfLib_SetInfo_BootClusterProtection
  ;-----------------------------------------------------------------------------
  ; Block type:   selflib command function
  ; C prototype:  u08  SelfLib_SetInfo_BootClusterProtection(void);
  ; C example:    my_status_u08 =  SelfLib_SetInfo_BootClusterProtection();
  ;-----------------------------------------------------------------------------
  ; Purpose:      sets the bootcluster-update-protection flag in the extra area
  ; Input:        -
  ; Output:       -
  ; Returned:     u08 - status code
  ;                   = 0x00(STS_NO_ERROR), normal
  ;                   = 0x05(STS_PARAMETER_ERROR), parameter error
  ;                   = 0x10(STS_PROTECTION_ERROR), protection error
  ;                   = 0x1A(STS_MRG10_ERROR), erase error
  ;                   = 0x1B(STS_MRG11_ERROR), internal verify error
  ;                   = 0x1F(STS_INTERRUPTED), write interrupted by user interrupt
  ;-----------------------------------------------------------------------------
  SelfLib_SetInfo_BootClusterProtection:        ;
      PUSH    BC                                ; push used register
      MOV     B,#MASK_BOOTCLUSTER_PROTECTION    ; B <- security information
      BR      SelfLib_SetInfo_common            ; goto common part of SetInfo
                                                ; -----------------------------------
#endif



; if any SetInfo command included unwrap the common part
#if (SI_SBC OR SI_CEP OR SI_BEP OR SI_WP OR SI_BCP)

  SelfLib_SetInfo_common:                         ; read actual security info
        MOV     A,#OPT_SECURITY_INFO              ; {
        MOV     (entry_ram+IDX_OPTION_BYTE),A     ;   entry_ram[IDX_OPTION_BYTE] <- security
        MOV     A,#CMD_GET_INFO                   ;   set command
        CALL    ASM_execute                       ;   execute the "get info" command
        CMP     A,#STS_NO_ERROR                   ; }
        BZ      SelfLib_SetInfo_continue          ; if (error) exit;
        POP     BC                                ; else
        RET                                       ; {

  SelfLib_SetInfo_continue:
        PUSH    HL                                ; save used register
        PUSH    AX                                ; ---------------------------
        MOVW    AX,(entry_ram+IDX_DATA_BUF_H)     ;
        MOVW    HL,AX                             ; HL <- address of data_buffer

        MOV     A,#0FFH                           ;
        MOV1    CY,[HL].0                         ;
        MOV1    A.1,CY                            ;
        MOV1    CY,[HL].1                         ;
        MOV1    A.2,CY                            ;
        MOV1    CY,[HL].2                         ;
        MOV1    A.3,CY                            ;
        MOV1    CY,[HL].4                         ;
        MOV1    A.5,CY                            ; A <- old security info

        AND     A,B                               ;
        MOV     [HL],A                            ; A <- new security info
                                                  ; ------------------------------------
        POP     AX                                ; restore used register
        MOV     A,#CMD_SET_INFO                   ;
        CALL    ASM_execute                       ; execute the "set info" command
                                                  ; -----------------------------
        POP    HL                                 ; restore used register
        POP    BC                                 ; restore used register
        RET
#endif


;-----------------------------------------------------------------------------
; Block type:   local assembler subroutine
; Purpose:      execution of a specified command
; Input:        A, selflib command code (defined in spl78k0_kx2_const.h)
; Output:       -
; Returned:     A, status_code (code defined in spl78k0_kx2_const.h)
;-----------------------------------------------------------------------------
ASM_execute:
    PUSH    PSW                               ; save user register bank context
    MOV     0FEE2H,A                          ; RB3.C <- command code RB(user).A
    SEL     RB3                               ; select RB3 for firmware operation
    MOVW    HL,#entry_ram                     ; RP3.HL <- &entry_ram
    DI                                        ;
    MOV     PFCMD,#0A5H                       ; ---------------------------
    MOV     FLPMC,#001H                       ;
    MOV     FLPMC,#0FEH                       ; set selfprogramming mode (A1)
    MOV     FLPMC,#001H                       ;
                                              ; ***************************
    CALL    08100H                            ; >>>> call the firmware <<<<
                                              ; ***************************
    MOV     PFCMD,#0A5H                       ;
    MOV     FLPMC,#000H                       ; set "normal mode" (A0)
    MOV     FLPMC,#0FFH                       ;
    MOV     FLPMC,#000H                       ; ---------------------------
    POP     PSW                               ; restore user RB and EI context
    MOV     A,0FEE3H                          ; RB(user).A <- status code (RB3.B)
    NOP                                       ; eventually interrupt services
    NOP                                       ; eventually interrupt services
    NOP                                       ; eventually interrupt services
    NOP                                       ; eventually interrupt services
    NOP                                       ; eventually interrupt services
    RET                                       ;

; zero-terminated driver information string
SPL_INFO:
    DB      SPL_DEVICE,'/',SPL_VERSION,0x00

  END





