//==============================================================================
// PROJECT      = K_Line - Taste it!-2
// MODULE       = init.c
// SHORT DESC.  = -
// DEVICE       = 78K0/KF2 (uPD78F0547D)
// VERSION      = 1.0
// DATE         = 19.10.2005
// LAST CHANGE  = -
// =============================================================================
// Description:  Initialization of Registers and Bit Variables
// =============================================================================
// By:          NEC Electronics (Europe) GmbH
//              Arcadiastrasse 10
//              D-40472 Duesseldorf
// =============================================================================

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "defines.h"
#include "lcd.h"

#include <intrinsics.h>

#include "io78f054780.h"

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
extern volatile unsigned char  bTimer1Flag;      // status Flag 16bit Timer one
extern volatile unsigned char  bTimer2Flag;      // status Flag 16bit Timer two
extern volatile unsigned char  bWatchTimer;      // Status Flag Watch Timer
extern volatile unsigned char  bKey3Flag;        // status Flag Key3
extern volatile unsigned char  bKey2Flag;        // status Flag Key2


//-----------------------------------------------------------------------------
// Module: vHardwareInit
// Description: This module initialize peripheral hardware.
//-----------------------------------------------------------------------------
void vHardwareInit(void)               // hardware inizialization
{
// clock generator setting
//------------------------
    OSCCTL = 0xF1;                     // external clock input for main and sub clock

    PCC  = 0x00;                       // use high speed mode fxp
    MOC  = 0x00;                       // enable external clock input mode

    MCM  = 0x03;                       // switch CPU to high speed

    IMS  = 0xCC;                       // select 1 KB internal High-Speed RAM and 128 KB ROM
    IXS  = 0x00;                       // select 6 KB internal expansion RAM

    OSTS = 0x02;                       // 2^15/fx ms wait after STOP release by interrupt

// port setting
//-------------
    PM5 = 0xF0;                       //set Port5 to output mode
    PM4 = 0x00;                       //set Port4 to output mode
    PM6 = 0x00;                       //set Port6 to output mode

// ext. Interrupt setting
// -------------------
    EGP    = 0x03;                     // setting ext.interrupt 0, 1: rising/falling edge
    EGN    = 0x03;                     //

    PMK0   = FALSE;                    // enable ext. interrupt 0 (Mask Flag Register)
    PMK1   = FALSE;                    // enable ext. interrupt 1 (Mask Flag Register)

// 16 Bit Timer setting
// --------------------
    INITTIMER1;
    INITTIMER2;

// Watch Timer setting
// --------------------

    STARTWATCHTIMER;
    WTMK = 0;                           // enable watch timer interrupt
}


//-----------------------------------------------------------------------------
// Module: vSoftwareInit
// Description: This module initialize variables.
//-----------------------------------------------------------------------------
void vSoftwareInit(void)
{
    bTimer1Flag  = FALSE;              // reset status flag 16bit Timer one
    bTimer2Flag  = FALSE;              // reset status flag 16bit Timer two
    bWatchTimer  = FALSE;              // reset status flag Watch Timer
    bKey3Flag    = FALSE;              // reset status flag Key3
    bKey2Flag    = FALSE;              // reset status flag Key2
}


