//==============================================================================
// PROJECT      = K_Line - Taste it!-2
// MODULE       = lcd.c
// SHORT DESC.  = -
// DEVICE       = 78K0/KF2 (uPD78F0547D)
// VERSION      = 1.0
// DATE         = 19.10.2005
// LAST CHANGE  = -
// =============================================================================
// Description:  LCD driver routines
// =============================================================================
// By:          NEC Electronics (Europe) GmbH
//              Arcadiastrasse 10
//              D-40472 Duesseldorf
// =============================================================================

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "defines.h"
#include "lcd.h"

#include <intrinsics.h>

#include "io78f054780.h"

//-----------------------------------------------------------------------------
// Module:   LCD_init
// Description: Initialization of LCD module
//-----------------------------------------------------------------------------
void LCD_init(void)
{
    LCD_inst(0x38);
    LCD_inst(0x38);
    LCD_inst(0x38);
    LCD_inst(0x06);
    LCD_inst(0x0c);
}

//-----------------------------------------------------------------------------
// Module:   LCD_string
// Description: Send character string to LCD module
//-----------------------------------------------------------------------------
void LCD_string(char const *point)
{
  while(point[0])
  {
      LCD_char(*point);
      *point++;
  }
}

//-----------------------------------------------------------------------------
// Module:   LCD_inst
// Description: Send instruction to LCD module
//-----------------------------------------------------------------------------
void LCD_inst(unsigned char data)
{
  LCD_busy();
  EE=0;
  RW=0;
  RS=0;
  __no_operation();
  __no_operation();
  EE=1;
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  DB=data;
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  EE=0;
 }

//-----------------------------------------------------------------------------
// Module:   LCD_data
// Description: Display value
//-----------------------------------------------------------------------------
void LCD_data(unsigned char data)
{
  LCD_busy();
  EE=0;
  RW=0;
  RS=1;
  __no_operation();
  __no_operation();
  EE=1;
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  DB=(data|0x30);
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  EE=0;
}

//-----------------------------------------------------------------------------
// Module:   LCD_char
// Description: Display single character
//-----------------------------------------------------------------------------
void LCD_char(unsigned char data)
{
  LCD_busy();
  EE=0;
  RW=0;
  RS=1;
  __no_operation();
  __no_operation();
  EE=1;
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  DB=data;
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  EE=0;
}

//-----------------------------------------------------------------------------
// Module:   LCD_busy
// Description: Check internal operation mode of LCD controller,
//              wait for busy flag equal zero
//-----------------------------------------------------------------------------
void LCD_busy(void)
{
  unsigned char BF=1;
  DB_mode=0xff;
  while(BF)
  {
    EE=0;
    RW=1;
    RS=0;
  __no_operation();
  __no_operation();
    EE=1;
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
    BF=DB7_busy;
  __no_operation();
  __no_operation();
    EE=0;
  __no_operation();
  __no_operation();
    RS=1;
    RW=0;
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  }
  DB_mode=0x00;
}

//-----------------------------------------------------------------------------
// Module:   LCD_cursor
// Description: Set display cursor
//-----------------------------------------------------------------------------
void LCD_cursor(unsigned char data)
{
  LCD_busy();
  EE=0;
  RW=0;
  RS=0;
  __no_operation();
  __no_operation();
  EE=1;
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  DB=(data|0x80);
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  EE=0;
}

//-----------------------------------------------------------------------------
// Module:  LCD_write_CGRAM
// Description: Write user defined character pattern to CGRAM
//-----------------------------------------------------------------------------
void LCD_write_CGRAM(unsigned char CGR_addr, char const *CGR_data)
{
 unsigned char i;
 for(i=0; i<8; i++)
  {
    LCD_inst((CGR_addr<<3)|0x40|i);
    LCD_char(*CGR_data);
    *CGR_data++;
  }
}
