//==============================================================================
// PROJECT      = K_Line - Taste it!-2
// MODULE       = timer_main.c
// SHORT DESC.  = -
// DEVICE       = 78K0/KF2 (uPD78F0547D)
// VERSION      = 1.0
// DATE         = 19.10.2005
// LAST CHANGE  = -
// =============================================================================
// Description: This sample program demonstrates a real time clock. After the
//              program is started the watch timer is initialized to generate
//              an exact clock reference, based on the 32.768 kHz sub-clock.
//              The actual time is displayed using the LCD module. By pressing
//              button SW2 / SW3 the time can be adjusted.
// =============================================================================
// By:          NEC Electronics (Europe) GmbH
//              Arcadiastrasse 10
//              D-40472 Duesseldorf
// =============================================================================

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "defines.h"
#include "lcd.h"

#include <intrinsics.h>

#include "io78f054780.h"

////-----------------------------------------------------------------------------
//// Option Byte
////-----------------------------------------------------------------------------
//#pragma location = "OPTBYTE"
//__root const unsigned char opbyte=0x00;
//
////---------------------------------------------------------------------------------
//// Security ID CODE: for OCD on-chip debugging only / 78K0 CPU module + QB-78K0MINI
////---------------------------------------------------------------------------------
//#pragma location = "SECUID"
//__root const unsigned char secuid[11]={0x02,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};

//-----------------------------------------------------------------------------
// Function prototyps
//-----------------------------------------------------------------------------
       void          vWait25        (unsigned char);  // n *  25ms delay
extern void          vHardwareInit  (void);           // initialization Hardware
extern void          vSoftwareInit  (void);           // initialization bit variables
extern void UART_SendChar (unsigned char ucData);
extern void UART_SendString (const char *ucpStr);

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
volatile unsigned char  bTimer1Flag;      // status Flag 16bit Timer one
volatile unsigned char  bTimer2Flag;      // status Flag 16bit Timer two
volatile unsigned char  bWatchTimer;      // Status Flag Watch Timer
volatile unsigned char  bKey3Flag;        // status Flag Key3
volatile unsigned char  bKey2Flag;        // status Flag Key2

unsigned char        menu;

__no_init unsigned char  setclock_done;
__no_init volatile struct {
          unsigned char sec;
          unsigned char min;
          unsigned char hour;
         }clock;

//-----------------------------------------------------------------------------
// Constants
//-----------------------------------------------------------------------------
const char *text01="NEC  K_Line ";
const char *text02="Taste it!-2 ";
const char *text03=" WatchTimer ";
const char *text04="    Demo    ";
const char *text05="----Time----";
const char *text06="  Set hour  ";
const char *text07=" Set minute ";
const char *text08=" Set second ";

//-----------------------------------------------------------------------------
// Module name: vWait25
// Description: This module delays the program for n * 50 ms.
//-----------------------------------------------------------------------------
void vWait25(unsigned char Number)
{
  bTimer2Flag = FALSE;                 // reset flag timer2
  STARTTIMER2;                         // start Timer2
  while(Number > 0) {
         while(bTimer2Flag == FALSE){  // wait for Timer Interrupt
          __no_operation();
          __no_operation();
        }
        bTimer2Flag = FALSE;           // reset status flag Timer2
        Number--;
    }
  STOPTIMER2;                          // stop Timer2
  return;
}

//-----------------------------------------------------------------------------
// Module name: vWait10
// Description: This module delays the program for 10 ms.
//-----------------------------------------------------------------------------
void vWait10(unsigned char Number)
{
  bTimer1Flag = FALSE;                 // reset flag timer1
  STARTTIMER1;                         // start Timer1
  while(Number > 0) {
        while(bTimer1Flag == FALSE){  // wait for Timer Interrupt
          __no_operation();
          __no_operation();
        }
        bTimer1Flag = FALSE;           // reset status flag Timer2
        Number--;
        }
  STOPTIMER1;                          // stop Timer1
  return;
}

//-----------------------------------------------------------------------------
// Module name: Output_result
// Description: Ouput of measured reaction time.
//-----------------------------------------------------------------------------
void Output_result(unsigned char data)
{
      unsigned char bcd0,bcd1;

      bcd0 = data / 10;
      bcd1 = data % 10;

      LCD_data(bcd0);
      LCD_data(bcd1);
}


//-----------------------------------------------------------------------------
// main function
//-----------------------------------------------------------------------------
void main( void )
{
    __disable_interrupt();             // disable all interrupts
    vHardwareInit();                   // peripheral settings
    vSoftwareInit();                   // variable Initialization
    LCD_init();                        // LCD Initialization
    LCD_inst(0x01);                    // clear display

    __enable_interrupt();              // enable all interrupts

    LCD_cursor(0x0);
    LCD_string(&text01[0]);
    LCD_cursor(0x40);
    LCD_string(&text02[0]);
    vWait25(80);                    // delay of 1000 ms
    LCD_cursor(0x0);
    LCD_string(&text03[0]);
    LCD_cursor(0x40);
    LCD_string(&text04[0]);
    vWait25(80);

    if(setclock_done!=0x5a)
    {
          clock.hour=0;
          clock.min=0;
          clock.sec=0;
    }
    LCD_inst(0x01);                    // clear display

    while(1)
    {
          LCD_cursor(0x42);
          Output_result(clock.hour);
          LCD_char(0x3A);
          Output_result(clock.min);
          LCD_char(0x3A);
          Output_result(clock.sec);
          switch(menu)
          {
              case(CLOCK):   LCD_cursor(0x00);
                             LCD_string(&text05[0]);
                             LCD_inst(0x0c);                    // blinking cusor off
                             break;

              case(SETHOUR): LCD_cursor(0x00);
                             LCD_string(&text06[0]);
                             LCD_inst(0x0d);                    // blinking cusor on
                             LCD_cursor(0x43);
                             if(bKey3Flag>=2)                   // Key SW3 pressed ?
                             {
                                   bKey3Flag=0;
                                   clock.hour++;
                                   if(clock.hour>=24) clock.hour=0;
                             }
                             break;

              case(SETMIN):  LCD_cursor(0x00);
                             LCD_string(&text07[0]);
                             LCD_inst(0x0d);                    // blinking cusor on
                             LCD_cursor(0x46);
                             if(bKey3Flag>=2)                   // Key SW3 pressed ?
                             {
                                   bKey3Flag=0;
                                   clock.min++;
                                   if(clock.min>=60) clock.min=0;
                             }
                             break;

              case(SETSEC):  LCD_cursor(0x00);
                             LCD_string(&text08[0]);
                             LCD_inst(0x0d);                    // blinking cusor on
                             LCD_cursor(0x49);
                             if(bKey3Flag>=2)                   // Key SW3 pressed ?
                             {
                                   bKey3Flag=0;
                                   clock.sec++;
                                   if(clock.sec>=60) clock.sec=0;
                             }
                             break;

          }
          if(bKey2Flag>=2)                 // Key SW2 pressed ?
          {
             bKey2Flag=0;
             menu++;                       // change menu
             if(menu==4)
             {
                 setclock_done=0x5a;
                 menu=0;
             }
          }
          vWait10(10);                     // wait for 100ms
    }
}

